import json
import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

OPCFOUNDATION_NAMESPACE = "http://opcfoundation.org/UA/"


def get_namespace_data_from_file(json_file_path: str) -> dict:
    """
    Get namespace data from JSON file
    Args:
        xml_file (str): The full filepath to the xml file to check

    Returns:
        A dictionary containing file namespace name and a list of required namespaces
        Example:
            {
                "name": "http://powerview.com/enterprise",
                "included_namespaces": {
                    "http://prediktor.no/PVTypes/",
                    "http://opcfoundation.org/UA/",
                }
            }
    """
    if not json_file_path.endswith("_parsed.json"):
        json_file_path += "_parsed.json"
    # In some NodeSet2 definition files the <Model> tag is not found
    # therefore using the common files name as well.
    opcua_namespace_data = {
        "name": OPCFOUNDATION_NAMESPACE,
        "included_namespaces": set(),
    }
    if json_file_path.endswith("Opc.Ua.NodeSet2.xml_parsed.json"):
        return opcua_namespace_data

    namespace_data = None
    namespace_uris_line = None

    with open(json_file_path, "r") as f:
        for line in f.readlines():
            line = json.loads(line)
            namespace_data = None
            if line["elem_type"] == "Models":
                for model_uri in line["uris"]:
                    if model_uri == OPCFOUNDATION_NAMESPACE:
                        namespace_data = opcua_namespace_data
                    else:
                        namespace_data = {
                            "name": model_uri,
                            "included_namespaces": {OPCFOUNDATION_NAMESPACE},
                        }
                    break  # get just the first Model tag (there should be no more Model tags)

            if namespace_data is not None:
                break

            if line["elem_type"] == "NamespaceUris":
                namespace_uris_line = line
                continue

    if namespace_data is None:
        message = f"Missing 'Model' tag in {json_file_path}"
        logger.error(message)
        raise ValueError(message)

    if namespace_uris_line is None:
        if namespace_data["name"] == OPCFOUNDATION_NAMESPACE:
            return namespace_data
        else:
            message = f"Missing 'NamespaceUris' tag in {json_file_path}"
            logger.error(message)
            raise ValueError(message)

    for namespace_uri_tag in namespace_uris_line["uris"]:
        if namespace_uri_tag != namespace_data["name"]:
            namespace_data["included_namespaces"].add(namespace_uri_tag)

    return namespace_data



import json
from typing import List

import lxml.etree as ET

from opcua_tools.json_parser.type_hints import (
    AliasesLine,
    ModelLine,
    ModelsLine,
    NameSpaceURIsLine,
    RequiredModelLine,
    UANodeSetLine,
)
from opcua_tools.ua_data_types import UANodeId
from opcua_tools.value_parser import parse_nodeid


def pre_process_xml_to_json(file_path):
    """
    Creates a JSON file which should have the same data as this XML file
    but in a more easily digestable format (JSONL).
    The goal is to separate the pre-processing of XML files (convert to processed JSON)
    from processing the data in XML files. This function is the pre-processing step.
    The pre-processing is very opinionated: each JSON line is formatted, so the next step
    which reads these JSON files can do it while using little memory and CPU time.
    """
    output_file_name = f"{file_path}_parsed.json"

    uanodeset_tag = "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}UANodeSet"
    namespace_uris_tag = (
        "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}NamespaceUris"
    )
    uri_tag = "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}Uri"
    models_tag = "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}Models"
    model_tag = "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}Model"
    required_model_tag = (
        "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}RequiredModel"
    )
    aliases_tag = "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}Aliases"
    alias_tag = "{http://opcfoundation.org/UA/2011/03/UANodeSet.xsd}Alias"

    tree = ET.parse(file_path)
    root = tree.getroot()

    with open(output_file_name, "w") as f:
        root_iter = root.iter()
        for elem in root_iter:
            if elem.tag == uanodeset_tag:
                line: UANodeSetLine = {"elem_type": "UANodeSet", "tag": uanodeset_tag}
                f.write(json.dumps(line) + "\n")
            elif elem.tag == namespace_uris_tag:
                line: NameSpaceURIsLine = {
                    "elem_type": "NamespaceUris",
                    "uris": [uri_elem.text for uri_elem in elem],
                }
                f.write(json.dumps(line) + "\n")
            elif elem.tag == model_tag:
                pass
            elif elem.tag == required_model_tag:
                pass
            elif elem.tag == uri_tag:
                pass
            elif elem.tag == models_tag:
                models: List[ModelLine] = []
                for model in elem:
                    req_models: List[RequiredModelLine] = []
                    for req_model in model.iterchildren():
                        req_models.append(
                            {
                                "uri": req_model.get("ModelUri"),
                                "publication_date": req_model.get("PublicationDate"),
                                "version": req_model.get("Version"),
                            }
                        )
                    models.append(
                        {
                            "uri": model.get("ModelUri"),
                            "publication_date": model.get("PublicationDate"),
                            "version": model.get("Version"),
                            "required_models": req_models,
                        }
                    )
                line: ModelsLine = {
                    "elem_type": "Models",
                    "uris": [model.get("ModelUri") for model in elem],
                    "models": models,
                }
                f.write(json.dumps(line) + "\n")
            elif elem.tag == alias_tag:
                pass
            elif elem.tag == aliases_tag:
                aliases = list()
                line: AliasesLine = {"elem_type": "Aliases", "aliases": aliases}
                for alias in elem:
                    res: UANodeId = parse_nodeid(alias.text)
                    aliases.append(
                        {
                            "name": alias.attrib["Alias"],
                            "nodeid": {
                                "namespace": res.namespace,
                                "type": res.nodeid_type.value,
                                "value": res.value,
                            },
                        }
                    )
                f.write(json.dumps(line) + "\n")
            else:
                break



import os.path

from definitions import get_project_root

from opcua_tools import nodeset_parser
from opcua_tools.json_parser import namespaces, parse


def test_get_namespace_data_should_process_opcua_file_regardless_file_name():
    example_xml_path = os.path.join(
        get_project_root(),
        "tests",
        "testdata",
        "invalid_example",
        "opcua_file_with_different_name",
        "OpcUA_cut.xml",
    )
    parse.pre_process_xml_to_json(example_xml_path)

    example_json_path = f"{example_xml_path}_parsed.json"

    ns_data = namespaces.get_namespace_data_from_file(example_json_path)

    assert ns_data == {
        "included_namespaces": set(),
        "name": "http://opcfoundation.org/UA/",
    }



from typing import List, TypedDict


class UANodeSetLine(TypedDict):
    elem_type: str
    tag: str


class NameSpaceURIsLine(TypedDict):
    elem_type: str
    uris: List[str]


class RequiredModelLine(TypedDict):
    publication_date: str
    uri: str
    version: str


class ModelLine(TypedDict):
    uri: str
    publication_date: str
    required_models: List[RequiredModelLine]
    version: str


class ModelsLine(TypedDict):
    elem_type: str
    uris: List[str]
    models: List[ModelLine]


class NodeIdLine(TypedDict):
    namespace: str
    type: str
    value: str


class AliasLine(TypedDict):
    name: str
    nodeid: NodeIdLine


class AliasesLine(TypedDict):
    elem_type: str
    aliases: List[AliasLine]
